#####################
# The power analysis is based on the data from the first experiment reported in
# 
# Belardinelli, A., Lohmann, J., Farn, A., & Butz, M. V. (2018). Mental space maps into the future. Cognition, 176, 65-73.
#
# The data is available from
#
# https://osf.io/8pgjn/
#
# The results regarding the verbal response times were used to perform a Monte Carlo simulation to
# determine the number of participants to obtain the sought three-way interaction between visual
# distractor, tactile stimulation, and bottle orientation with a power of .9. This was operationalized
# in terms of the proportion of runs in which the simulation yielded a significant interaction (p < .05).
# The results of the simulations are included in the comments, a sample size of 20 seemed sufficient.
#####################

# set this to the path were the data is located, remember to add two backslashes on Windos systems, e.g.
# C:\\Users\\Documents
setwd("")

# read the data
data <- read.table("Data.dat", header = TRUE)
# just to check
testdata <- aggregate(RT ~ ParticipantID*BottleOrientation*SOACondition*Finger*Distractor, data, mean)
testdata <- droplevels(testdata)

summary(testdata)

testdata$ParticipantID <- factor(testdata$ParticipantID)
levels(testdata$ParticipantID)
length(levels(testdata$ParticipantID))
# there should be 3 (SOA) x 2 (orientation) x 2 (distractor side) x 2 (finger) = 24 data points per participant
aggregate(RT ~ ParticipantID, testdata, length)
# this package is required for the ANOVA
library("ez")
# this is our distribution data that we use for the sampling
aggregatedData <- aggregate(RT ~ BottleOrientation*SOACondition*Finger*Distractor, data, mean)
aggregatedData <- droplevels(aggregatedData)
# we also need the standard deviations
sds <- aggregate(RT ~ BottleOrientation*SOACondition*Finger*Distractor, data, function(x) {sd(x)})
sds <- droplevels(sds)
# which we add to the data structure
aggregatedData <- cbind(aggregatedData, sds[, 5])

#### simulation

numberOfParticipants <- 20
# number of simulation runs
repetitions <- 1000
# results are stored here: whether the interaction was significant as well as the according effect size
sim.results <- data.frame(rep(0, repetitions), rep(0, repetitions))
colnames(sim.results) <- c("SigIA", "EtaIA")

for (run in 1:repetitions)
{
  sampleData <- data.frame(rep(0, 
                               nrow(aggregatedData) * numberOfParticipants), # 24 lines per participant
                           rep(c('rotated', 'upright'),  # bottle orientations
                               nrow(aggregatedData) * numberOfParticipants / 2),
                           rep(c('SOA1', 'SOA1', 'SOA2', 'SOA2', 'SOA3', 'SOA3'), # SOAs
                               nrow(aggregatedData) * numberOfParticipants / 6),
                           rep(c('Index', 'Index', 'Index', 'Index', 'Index', 'Index', 'Thumb', 'Thumb', 'Thumb', 'Thumb', 'Thumb', 'Thumb'), # stimulated finger
                               nrow(aggregatedData) * numberOfParticipants / 12),
                           rep(c('left', 'left', 'left', 'left', 'left', 'left', 'left', 'left', 'left', 'left', 'left', 'left', 'right', 'right', 'right', 'right', 'right', 'right', 'right', 'right', 'right', 'right', 'right', 'right'), # side of the visual distractor
                               nrow(aggregatedData) * numberOfParticipants / 24),
                           rep(0, nrow(aggregatedData) * numberOfParticipants)) # the last column contains the simulated verbal RTs
  colnames(sampleData) <- c("ID", "BottleOrientation", "SOACondition", "Finger", "Distractor", "RT")
  # data sampling
  for (id in 1:numberOfParticipants)
  {
    for (i in 1:nrow(aggregatedData))
    {
      sampleData[(id - 1) * nrow(aggregatedData) + i, 1] <- id
      # we sample by assuming a normal distribution
      sampleData[(id - 1) * nrow(aggregatedData) + i, 6] <- rnorm(1, aggregatedData[i, 5], aggregatedData[i, 6])
    }
  }
  # analysis of simulated data
  sampleData$ID <- factor(sampleData$ID)
  sampleData$BottleOrientation <- factor(sampleData$BottleOrientation)
  sampleData$SOACondition <- factor(sampleData$SOACondition)
  sampleData$Finger <- factor(sampleData$Finger)
  sampleData$Distractor <- factor(sampleData$Distractor)
  
  anova_rta <- ezANOVA(data=sampleData, dv = .(RT), wid = .(ID), within = .(BottleOrientation,SOACondition,Finger,Distractor), detailed = TRUE)
  # if an effect was significant, we add a 1 to results
  if (anova_rta$ANOVA$p[14] < .05)
  {
    sim.results[run, 1] <- 1
  }
  # we store the effect sizes
  partial_etas <- anova_rta$ANOVA$SSn / (anova_rta$ANOVA$SSn + anova_rta$ANOVA$SSd)
  sim.results[run, 2] <- partial_etas[14]
}
# this gives us the result of the run
colMeans(sim.results)

# 20 participants : 
#       SigIA         EtaIA 
#   0.8770000     0.3585906 
#             +/- 0.00907849
#
# 15 participants :
#       SigIA         EtaIA
#    0.732000     0.3680475 
#             +/- 0.0109548
# 10 participants : 
#       SigIA         EtaIA
#    0.4830000    0.3645878
#             +/- 0.0131558
# a confidence interval for the effect size 
qnorm(0.975) * sd(sim.results[, 2]) / sqrt(repetitions)